@@ -1,10 +1,10 @@
 /*
- * $Id: avrcore.c,v 1.77 2003/12/03 18:37:59 troth Exp $
+ * $Id: avrcore.c,v 1.81 2005/01/13 20:27:59 zfrdh Exp $
  *
  ****************************************************************************
  *
  * simulavr - A simulator for the Atmel AVR family of microcontrollers.
- * Copyright (C) 2001, 2002, 2003  Theodore A. Roth
+ * Copyright (C) 2001, 2002, 2003, 2004  Theodore A. Roth
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,6 +58,7 @@
 #include "display.h"
 #include "decoder.h"
 #include "sig.h"
+#include "devsupp.h"
 
 /** \brief Flag for enabling output of instruction debug messages. */
 int global_debug_inst_output = 0;
@@ -342,6 +343,8 @@ irq_get_head_vector (DList *head)
  *
 \***************************************************************************/
 
+static void avr_core_construct (AvrCore *core, DevSuppDefn *dev);
+
 /** \name AvrCore handling methods */
 
 /*@{*/
@@ -349,25 +352,35 @@ irq_get_head_vector (DList *head)
 /** \brief Allocate a new AvrCore object. */
 
 AvrCore *
-avr_core_new (int flash_sz, int PC_sz, StackType stack_type, int stack_sz,
-              int vtab_idx)
+avr_core_new (char *dev_name)
 {
-    AvrCore *core;
+    AvrCore *core = NULL;
+    DevSuppDefn *dev = dev_supp_lookup_device (dev_name);
 
-    core = avr_new (AvrCore, 1);
-    avr_core_construct (core, flash_sz, PC_sz, stack_type, stack_sz,
-                        vtab_idx);
-    class_overload_destroy ((AvrClass *)core, avr_core_destroy);
+    if (dev)
+    {
+        fprintf (stderr, "\nSimulating a %s device.\n\n", dev_name);
+
+        core = avr_new (AvrCore, 1);
+        avr_core_construct (core, dev);
+        class_overload_destroy ((AvrClass *)core, avr_core_destroy);
+    }
 
     return core;
 }
 
 /** \brief Constructor for the AvrCore class. */
 
-void
-avr_core_construct (AvrCore *core, int flash_sz, int PC_sz,
-                    StackType stack_type, int stack_sz, int vtab_idx)
+static void
+avr_core_construct (AvrCore *core, DevSuppDefn *dev)
 {
+    int flash_sz = dev_supp_get_flash_sz (dev);
+    int PC_sz = dev_supp_get_PC_sz (dev);
+    int stack_sz = dev_supp_get_stack_sz (dev);
+    int sram_sz = dev_supp_get_sram_sz (dev);
+    int vtab_idx = dev_supp_get_vtab_idx (dev);
+    int addr;
+
     if (core == NULL)
         avr_error ("passed null ptr");
 
@@ -379,11 +392,8 @@ avr_core_construct (AvrCore *core, int flash_sz, int PC_sz,
     core->PC_size = PC_sz;
     core->PC_max = flash_sz / 2; /* flash_sz is in bytes, need number of
                                     words here */
-    core->sreg = sreg_new ();
+
     core->flash = flash_new (flash_sz);
-    core->gpwr = gpwr_new ();
-    core->rampz = rampz_new ();
-    core->mem = mem_new ();
 
     core->breakpoints = NULL;
 
@@ -397,34 +407,87 @@ avr_core_construct (AvrCore *core, int flash_sz, int PC_sz,
     core->clk_cb = NULL;
     core->async_cb = NULL;
 
-    switch (stack_type)
+    /* FIXME: hack to get it to compile. */
+    if (dev_supp_has_ext_io_reg (dev))
+        core->mem = mem_new (0x1f, 0xff, 0xff + sram_sz, 0xffff);
+    else
+        core->mem = mem_new (0x1f, 0x5f, 0x5f + sram_sz, 0xffff);
+
+    /* Attach the gpwr's to the memory bus. */
+
+    core->gpwr = gpwr_new ();
+    for (addr = 0; addr < 0x20; addr++)
     {
-        case STACK_HARDWARE:
-            core->stack = (Stack *)hwstack_new (stack_sz);
-            break;
-        case STACK_MEMORY:
-            core->stack = (Stack *)memstack_new (core->mem);
-            break;
-        default:
-            avr_error ("Bad stack type.\n");
+        static char *reg_name[] = { "r00", "r01", "r02", "r03", "r04", "r05",
+                                    "r06", "r07", "r08", "r09", "r10", "r11",
+                                    "r12", "r13", "r14", "r15", "r16", "r17",
+                                    "r18", "r19", "r20", "r21", "r22", "r23",
+                                    "r24", "r25", "r26", "r27", "r28", "r29",
+                                    "r30", "r31" };
+
+        avr_core_attach_vdev (core, addr, reg_name[addr],
+                              (VDevice *)core->gpwr, 0, 0, 0xff, 0xff);
     }
 
-    /* Since these are being stored in two places, we need to ref em */
+    dev_supp_attach_io_regs (core, dev);
 
-    avr_core_attach_vdev (core, (VDevice *)core->sreg);
+    /* Set up the stack. */
+
+    if (stack_sz)
+    {
+        core->stack = (Stack *)hwstack_new (stack_sz);
+    }
+    else
+    {
+        /* Assuming that SPL is always at 0x5d. */
+
+        core->stack = (Stack *)memstack_new (core->mem, 0x5d);
+    }
+
+    /* Assuming the SREG is always at 0x5f. */
+
+    core->sreg = (SREG *)avr_core_get_vdev_by_addr (core, 0x5f);
     class_ref ((AvrClass *)core->sreg);
 
-    avr_core_attach_vdev (core, (VDevice *)core->rampz);
-    class_ref ((AvrClass *)core->rampz);
+    /* Assuming that RAMPZ is always at 0x5b. If the device doesn't support
+       RAMPZ, install a NULL pointer. */
 
-    avr_core_attach_vdev (core, (VDevice *)core->gpwr);
-    class_ref ((AvrClass *)core->gpwr);
+    core->rampz = (RAMPZ *)avr_core_get_vdev_by_addr (core, 0x5b);
+    if (core->rampz)
+        class_ref ((AvrClass *)core->rampz);
+
+    /* Attach the internal sram to the memory bus if needed. */
+
+    if (sram_sz)
+    {
+        int base;
+        VDevice *sram;
+
+        if (dev_supp_has_ext_io_reg (dev))
+            base = SRAM_EXTENDED_IO_BASE;
+        else
+            base = SRAM_BASE;
+
+        core->sram = sram_new (base, sram_sz);
+        sram = (VDevice *)core->sram;
+
+        avr_message ("attach: Internal SRAM from 0x%04x to 0x%04x\n", base,
+                     (base + sram_sz - 1));
+
+        for (addr = base; addr < (base + sram_sz); addr++)
+        {
+            avr_core_attach_vdev (core, addr, "Internal SRAM", sram, 0, 0,
+                                  0xff, 0xff);
+        }
+    }
+    else
+    {
+        core->sram = NULL;
+    }
 
     /* Initialize the decoder lookup table */
-    decode_init_lookup_table ();
 
-    /* Send initial clock cycles to display */
-    display_clock (0);
+    decode_init_lookup_table ();
 }
 
 /**
@@ -460,14 +523,12 @@ void
 avr_core_get_sizes (AvrCore *core, int *flash, int *sram, int *sram_start,
                     int *eeprom)
 {
-    VDevice *dev;
     *flash = flash_get_size (core->flash);
 
-    dev = mem_get_vdevice_by_name (core->mem, "SRAM");
-    if (dev)
+    if (core->sram)
     {
-        *sram = sram_get_size ((SRAM *)dev);
-        *sram_start = sram_get_base ((SRAM *)dev);
+        *sram = sram_get_size (core->sram);
+        *sram_start = sram_get_base (core->sram);
     }
     else
     {
@@ -475,15 +536,17 @@ avr_core_get_sizes (AvrCore *core, int *flash, int *sram, int *sram_start,
         *sram_start = 0;
     }
 
-    dev = mem_get_vdevice_by_name (core->mem, "EEProm");
-    if (dev)
-        *eeprom = eeprom_get_size ((EEProm *)dev);
+    if (core->eeprom)
+        *eeprom = eeprom_get_size (core->eeprom);
     else
         *eeprom = 0;
 }
 
 /** \brief Attach a virtual device into the Memory. */
-extern inline void avr_core_attach_vdev (AvrCore *core, VDevice *dev);
+extern inline void avr_core_attach_vdev (AvrCore *core, uint16_t addr,
+                                         char *name, VDevice *vdev,
+                                         int flags, uint8_t reset_value,
+                                         uint8_t rd_mask, uint8_t wr_mask);
 
 /** \brief Returns the \c VDevice with the name \a name. */
 extern inline VDevice *avr_core_get_vdev_by_name (AvrCore *core, char *name);
@@ -1066,9 +1129,16 @@ avr_core_run (AvrCore *core)
     run_time = get_program_time () - start_time;
 
     signal_watch_stop (SIGINT);
-
+    
+    /* avoid division by zero below */
+    if (run_time == 0) run_time = 1;
+    
     avr_message ("Run time was %lld.%03lld seconds.\n", run_time / 1000,
                  run_time % 1000);
+
+    if (run_time == 0)
+         run_time = 1;          /* Avoid division by zero. */
+
     avr_message ("Executed %lld instructions.\n", cnt);
     avr_message ("   %lld insns/sec\n", (cnt * 1000) / run_time);
     avr_message ("Executed %lld clock cycles.\n", avr_core_CK_get (core));
@@ -1122,14 +1192,14 @@ avr_core_reset (AvrCore *core)
  *
  */
 void
-avr_core_add_ext_rd_wr (AvrCore *core, int port_id, PortFP_ExtRd ext_rd,
+avr_core_add_ext_rd_wr (AvrCore *core, int addr, PortFP_ExtRd ext_rd,
                         PortFP_ExtWr ext_wr)
 {
-    Port *p = (Port *)mem_get_vdevice_by_name (core->mem, name_PORT[port_id]);
+    Port *p = (Port *)mem_get_vdevice_by_addr (core->mem, addr);
 
     if (p == NULL)
     {
-        avr_warning ("Device does not have %s.\n", name_PORT[port_id]);
+        avr_warning ("Device does not have vdevice at 0x%04x.\n", addr);
         return;
     }
 
