@@ -1,10 +1,10 @@
 /*
- * $Id: devsupp.h,v 1.6 2003/12/01 07:35:52 troth Exp $
+ * $Id: devsupp.h,v 1.7 2004/01/30 07:09:56 troth Exp $
  *
  ****************************************************************************
  *
  * simulavr - A simulator for the Atmel AVR family of microcontrollers.
- * Copyright (C) 2001, 2002, 2003  Theodore A. Roth
+ * Copyright (C) 2001, 2002, 2003, 2004  Theodore A. Roth
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,18 @@
 #ifndef SIM_DEVSUPP_H
 #define SIM_DEVSUPP_H
 
+typedef struct _DevSuppDefn DevSuppDefn;
+
 extern void dev_supp_list_devices (FILE * fp);
-extern AvrCore *dev_supp_create_core (char *dev_name);
+extern DevSuppDefn *dev_supp_lookup_device (char *dev_name);
+extern void dev_supp_attach_io_regs (AvrCore *core, DevSuppDefn *dev);
+
+extern int dev_supp_get_flash_sz (DevSuppDefn *dev);
+extern int dev_supp_get_sram_sz (DevSuppDefn *dev);
+extern int dev_supp_get_PC_sz (DevSuppDefn *dev);
+extern int dev_supp_get_stack_sz (DevSuppDefn *dev);
+extern int dev_supp_get_vtab_idx (DevSuppDefn *dev);
+
+extern int dev_supp_has_ext_io_reg (DevSuppDefn *dev);
 
 #endif /* SIM_DEVSUPP_H */
