@@ -1,5 +1,5 @@
 /*
- * $Id: gdbserver.c,v 1.49 2003/12/02 03:56:45 troth Exp $
+ * $Id: gdbserver.c,v 1.50 2004/10/19 18:43:50 zfrdh Exp $
  *
  ****************************************************************************
  *
@@ -426,7 +426,7 @@ gdb_extract_hex_num (char **pkt, char stop)
         if (i > max_shifts)
             avr_error ("number too large");
 
-        num = (num << (i * 4)) | hex2nib (*p);
+        num = (num << 4)  | hex2nib (*p);
         i++;
         p++;
     }
