@@ -1,10 +1,10 @@
 /*
- * $Id: intvects.c,v 1.16 2003/12/02 20:28:27 troth Exp $
+ * $Id: intvects.c,v 1.17 2004/02/26 07:22:15 troth Exp $
  *
  ****************************************************************************
  *
  * simulavr - A simulator for the Atmel AVR family of microcontrollers.
- * Copyright (C) 2001, 2002, 2003  Theodore A. Roth
+ * Copyright (C) 2001, 2002, 2003, 2004  Theodore A. Roth
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -442,21 +442,6 @@ static IntVectTable vtab_at43usb325 = {
                         0x18, 0x00, NO_BIT, NO_BIT } /* TODO */
 };
 
-/* supports 324
-   NOTE: The vector addresses are not sequential. */
-static IntVectTable vtab_at43usb324 = {
-    .RESET          = { "IRQ_RESET",
-                        0x00, 0x00, NO_BIT, NO_BIT },
-    .INT0           = { "IRQ_INT0",
-                        0x02, 0x00, { 0x5b, 1<<6 }, { 0x5a, 1<<6 } },
-    .INT1           = { "IRQ_INT1",
-                        0x04, 0x00, { 0x5b, 1<<7 }, { 0x5a, 1<<7 } },
-    .TIMER0_OVF     = { "IRQ_TIMER0_OVF",
-                        0x0e, 0x00, { 0x59, 1<<1 }, { 0x58, 1<<1 } },
-    .USB_HW         = { "IRQ_USB_HW",
-                        0x18, 0x00, NO_BIT, NO_BIT } /* TODO */
-};
-
 /* supports 326
    NOTE: The vector addresses are not sequential. */
 static IntVectTable vtab_at43usb326 = {
@@ -488,7 +473,6 @@ IntVectTable *global_vtable_list[] = {
     &vtab_at43usb355,
     &vtab_at43usb320,
     &vtab_at43usb325,
-    &vtab_at43usb324,
     &vtab_at43usb326,
     NULL
 };
