@@ -1,10 +1,10 @@
 /*
- * $Id: timers.c,v 1.13 2003/12/02 08:25:00 troth Exp $
+ * $Id: timers.c,v 1.15 2004/03/13 19:55:34 troth Exp $
  *
  ****************************************************************************
  *
  * simulavr - A simulator for the Atmel AVR family of microcontrollers.
- * Copyright (C) 2001, 2002, 2003  Theodore A. Roth
+ * Copyright (C) 2001, 2002, 2003, 2004  Theodore A. Roth
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "avrerror.h"
 #include "avrmalloc.h"
@@ -60,52 +61,61 @@
 
 #ifndef DOXYGEN
 
+/* *INDENT-OFF* */
+
 Timer16Def global_timer16_defs[] = {
     {
-     .timer_name = "Timer1",
-     .tcnth_name = "TCNT1H",
-     .tcntl_name = "TCNT1L",
-     .tccra_name = "TCCR1A",
-     .tccrb_name = "TCCR1B",
-     .base = 0x4c,
-     .tof = bit_TOV1,
-     .ocf_a = bit_OCF1A,
-     .ocf_b = bit_OCF1B,
-     .ocf_c = 8}
+        .timer_name = "Timer1",
+        .tcnth_name = "TCNT1H",
+        .tcntl_name = "TCNT1L",
+        .tccra_name = "TCCR1A",
+        .tccrb_name = "TCCR1B",
+        .base = 0x4c,
+        .tof = bit_TOV1,
+        .ocf_a = bit_OCF1A,
+        .ocf_b = bit_OCF1B,
+        .ocf_c = 8
+    }
 };
 
 OCReg16Def global_ocreg16_defs[] = {
     {
-     .ocrdev_name = "OCR1A",
-     .ocrl_name = "OCR1AL",
-     .ocrh_name = "OCR1AH",
-     .base = 0x4a},
+        .ocrdev_name = "OCR1A",
+        .ocrl_name = "OCR1AL",
+        .ocrh_name = "OCR1AH",
+        .base = 0x4a
+    },
     {
-     .ocrdev_name = "OCR1B",
-     .ocrl_name = "OCR1BL",
-     .ocrh_name = "OCR1BH",
-     .base = 0x48},
+        .ocrdev_name = "OCR1B",
+        .ocrl_name = "OCR1BL",
+        .ocrh_name = "OCR1BH",
+        .base = 0x48
+    },
     {
-     .ocrdev_name = "OCR1C",
-     .ocrl_name = "OCR1CL",
-     .ocrh_name = "OCR1CH",
-     .base = 0x78},
+        .ocrdev_name = "OCR1C",
+        .ocrl_name = "OCR1CL",
+        .ocrh_name = "OCR1CH",
+        .base = 0x78},
     {
-     .ocrdev_name = "OCR3A",
-     .ocrl_name = "OCR3AL",
-     .ocrh_name = "OCR3A",
-     .base = 0x86},
+        .ocrdev_name = "OCR3A",
+        .ocrl_name = "OCR3AL",
+        .ocrh_name = "OCR3A",
+        .base = 0x86
+    },
     {
-     .ocrdev_name = "OCR3B",
-     .ocrl_name = "OCR3BL",
-     .ocrh_name = "OCR3BH",
-     .base = 0x84},
+        .ocrdev_name = "OCR3B",
+        .ocrl_name = "OCR3BL",
+        .ocrh_name = "OCR3BH",
+        .base = 0x84
+    },
     {
-     .ocrdev_name = "OCR3C",
-     .ocrl_name = "OCR3CL",
-     .ocrh_name = "OCR3CH",
-     .base = 0x82}
+        .ocrdev_name = "OCR3C",
+        .ocrl_name = "OCR3CL",
+        .ocrh_name = "OCR3CH",
+        .base = 0x82
+    }
 };
+/* *INDENT-ON* */
 
 #endif /* not DOXYGEN */
 
@@ -115,21 +125,33 @@ OCReg16Def global_ocreg16_defs[] = {
  *
 \****************************************************************************/
 
+static void timer_iadd_addr (VDevice *vdev, int addr, char *name,
+                             int rel_addr, void *data);
 static uint8_t timer_intr_read (VDevice *dev, int addr);
 static void timer_intr_write (VDevice *dev, int addr, uint8_t val);
 static void timer_intr_reset (VDevice *dev);
-static char *timer_intr_reg_name (VDevice *dev, int addr);
 static int timer_intr_cb (uint64_t time, AvrClass *data);
 
 /** \brief Allocate a new timer interrupt */
 
+VDevice *
+timer_int_create (int addr, char *name, int rel_addr, void *data)
+{
+    uint8_t *func_mask = (uint8_t *) data;
+    if (data)
+        return (VDevice *)timer_intr_new (addr, name, *func_mask);
+    else
+        avr_error ("Attempted timer interrupt create with NULL data pointer");
+    return 0;
+}
+
 TimerIntr_T *
-timer_intr_new (uint8_t func_mask)
+timer_intr_new (int addr, char *name, uint8_t func_mask)
 {
     TimerIntr_T *ti;
 
     ti = avr_new (TimerIntr_T, 1);
-    timer_intr_construct (ti, func_mask);
+    timer_intr_construct (ti, addr, name, func_mask);
     class_overload_destroy ((AvrClass *)ti, timer_intr_destroy);
 
     return ti;
@@ -138,22 +160,45 @@ timer_intr_new (uint8_t func_mask)
 /** \brief Constructor for timer interrupt object. */
 
 void
-timer_intr_construct (TimerIntr_T *ti, uint8_t func_mask)
+timer_intr_construct (TimerIntr_T *ti, int addr, char *name,
+                      uint8_t func_mask)
 {
-    char *name = "TimerIntr";
-
     if (ti == NULL)
         avr_error ("passed null ptr");
 
-    vdev_construct ((VDevice *)ti, name, TIMER_INTR_BASE, TIMER_INTR_SIZE,
-                    timer_intr_read, timer_intr_write, timer_intr_reset,
-                    timer_intr_reg_name);
+    vdev_construct ((VDevice *)ti, timer_intr_read, timer_intr_write,
+                    timer_intr_reset, timer_iadd_addr);
 
     ti->func_mask = func_mask;
 
+    timer_iadd_addr ((VDevice *)ti, addr, name, 0, NULL);
+
     timer_intr_reset ((VDevice *)ti);
 }
 
+static void
+timer_iadd_addr (VDevice *vdev, int addr, char *name, int rel_addr,
+                 void *data)
+{
+    TimerIntr_T *ti = (TimerIntr_T *)vdev;
+
+    if (strncmp ("TIFR", name, 4) == 0)
+    {
+        ti->tifr_addr = addr;
+    }
+
+    else if (strncmp ("TIMSK", name, 5) == 0)
+    {
+        ti->timsk_addr = addr;
+    }
+
+    else
+    {
+        avr_error ("invalid Timer Interrupt register name: '%s' @ 0x%04x",
+                   name, addr);
+    }
+}
+
 /** \brief Destructor for timer interrupt object. */
 
 void
@@ -170,15 +215,21 @@ timer_intr_read (VDevice *dev, int addr)
 {
     TimerIntr_T *ti = (TimerIntr_T *)dev;
 
-    switch (addr - vdev_get_base (dev))
+    if (addr == ti->timsk_addr)
     {
-        case TIMER_INTR_TIMSK_ADDR:
-            return (ti->timsk & ti->func_mask);
-        case TIMER_INTR_TIFR_ADDR:
-            return (ti->tifr & ti->func_mask);
-        default:
-            avr_error ("Bad address: 0x%04x", addr);
+        return (ti->timsk & ti->func_mask);
     }
+
+    else if (addr == ti->tifr_addr)
+    {
+        return (ti->tifr & ti->func_mask);
+    }
+
+    else
+    {
+        avr_error ("Bad address: 0x%04x", addr);
+    }
+
     return 0;                   /* will never get here */
 }
 
@@ -186,10 +237,9 @@ static void
 timer_intr_write (VDevice *dev, int addr, uint8_t val)
 {
     TimerIntr_T *ti = (TimerIntr_T *)dev;
-    int offset = addr - vdev_get_base (dev);
     CallBack *cb;
 
-    if (offset == TIMER_INTR_TIMSK_ADDR)
+    if (addr == ti->timsk_addr)
     {
         ti->timsk = (val & ti->func_mask);
         if (ti->timsk == 0)
@@ -205,10 +255,12 @@ timer_intr_write (VDevice *dev, int addr, uint8_t val)
             avr_core_async_cb_add ((AvrCore *)vdev_get_core (dev), cb);
         }
     }
-    else if (offset == TIMER_INTR_TIFR_ADDR)
+
+    else if (addr == ti->tifr_addr)
     {
-        ti->tifr = (val & ti->func_mask);
+        ti->tifr &= ~(val & ti->func_mask);
     }
+
     else
     {
         avr_error ("Bad address: 0x%04x", addr);
@@ -226,21 +278,6 @@ timer_intr_reset (VDevice *dev)
     ti->tifr = 0;
 }
 
-static char *
-timer_intr_reg_name (VDevice *dev, int addr)
-{
-    switch (addr - vdev_get_base (dev))
-    {
-        case TIMER_INTR_TIMSK_ADDR:
-            return "TIMSK";
-        case TIMER_INTR_TIFR_ADDR:
-            return "TIFR";
-        default:
-            avr_error ("Bad address: 0x%04x", addr);
-    }
-    return NULL;                /* will never get here */
-}
-
 static int
 timer_intr_cb (uint64_t time, AvrClass *data)
 {
@@ -300,21 +337,28 @@ timer_intr_cb (uint64_t time, AvrClass *data)
  *
 \****************************************************************************/
 
+static void timer0_add_addr (VDevice *vdev, int addr, char *name,
+                             int rel_addr, void *data);
 static uint8_t timer0_read (VDevice *dev, int addr);
 static void timer0_write (VDevice *dev, int addr, uint8_t val);
 static void timer0_reset (VDevice *dev);
-static char *timer0_reg_name (VDevice *dev, int addr);
 static int timer0_clk_incr_cb (uint64_t ck, AvrClass *data);
 
 /** \brief Allocate a new timer/counter 0. */
 
+VDevice *
+timer0_create (int addr, char *name, int rel_addr, void *data)
+{
+    return (VDevice *)timer0_new (addr, name, rel_addr);
+}
+
 Timer0_T *
-timer0_new (void)
+timer0_new (int addr, char *name, int rel_addr)
 {
     Timer0_T *timer;
 
     timer = avr_new (Timer0_T, 1);
-    timer0_construct (timer);
+    timer0_construct (timer, addr, name, rel_addr);
     class_overload_destroy ((AvrClass *)timer, timer0_destroy);
 
     return timer;
@@ -323,16 +367,17 @@ timer0_new (void)
 /** \brief Constructor for timer/counter 0 object. */
 
 void
-timer0_construct (Timer0_T *timer)
+timer0_construct (Timer0_T *timer, int addr, char *name, int rel_addr)
 {
-    char *name = "Timer0";
-
     if (timer == NULL)
         avr_error ("passed null ptr");
 
-    vdev_construct ((VDevice *)timer, name, TIMER0_BASE, TIMER0_SIZE,
-                    timer0_read, timer0_write, timer0_reset, timer0_reg_name);
+    vdev_construct ((VDevice *)timer, timer0_read, timer0_write, timer0_reset,
+                    timer0_add_addr);
 
+    timer0_add_addr ((VDevice *)timer, addr, name, 0, NULL);
+    if (rel_addr)
+        timer->related_addr = rel_addr;
     timer0_reset ((VDevice *)timer);
 }
 
@@ -347,20 +392,44 @@ timer0_destroy (void *timer)
     vdev_destroy (timer);
 }
 
+static void
+timer0_add_addr (VDevice *vdev, int addr, char *name, int rel_addr,
+                 void *data)
+{
+    Timer0_T *ti = (Timer0_T *)vdev;
+
+    if (strncmp ("TCNT", name, 4) == 0)
+    {
+        ti->tcnt_addr = addr;
+    }
+
+    else if (strncmp ("TCCR", name, 4) == 0)
+    {
+        ti->tccr_addr = addr;
+    }
+
+    else
+    {
+        avr_error ("invalid Timer register name: '%s' @ 0x%04x", name, addr);
+    }
+}
+
 static uint8_t
 timer0_read (VDevice *dev, int addr)
 {
     Timer0_T *timer = (Timer0_T *)dev;
 
-    switch (addr - vdev_get_base (dev))
+    if (addr == timer->tcnt_addr)
+        return timer->tcnt;
+
+    else if (addr == timer->tccr_addr)
+        return timer->tccr;
+
+    else
     {
-        case TIMER0_TCNT_ADDR:
-            return timer->tcnt;
-        case TIMER0_TCCR_ADDR:
-            return timer->tccr;
-        default:
-            avr_error ("Bad address: 0x%04x", addr);
+        avr_error ("Bad address: 0x%04x", addr);
     }
+
     return 0;                   /* will never get here */
 }
 
@@ -368,14 +437,14 @@ static void
 timer0_write (VDevice *dev, int addr, uint8_t val)
 {
     Timer0_T *timer = (Timer0_T *)dev;
-    int offset = addr - vdev_get_base (dev);
     CallBack *cb;
 
-    if (offset == TIMER0_TCNT_ADDR)
+    if (addr == timer->tcnt_addr)
     {
         timer->tcnt = val;
     }
-    else if (offset == TIMER0_TCCR_ADDR)
+
+    else if (addr == timer->tccr_addr)
     {
         /*
          * When the user writes toe TCCR, a callback is installed for either
@@ -431,6 +500,7 @@ timer0_write (VDevice *dev, int addr, uint8_t val)
                                  cb);
         }
     }
+
     else
     {
         avr_error ("Bad address: 0x%04x", addr);
@@ -451,21 +521,6 @@ timer0_reset (VDevice *dev)
     timer->divisor = 0;
 }
 
-static char *
-timer0_reg_name (VDevice *dev, int addr)
-{
-    switch (addr - vdev_get_base (dev))
-    {
-        case TIMER0_TCNT_ADDR:
-            return "TCNT0";
-        case TIMER0_TCCR_ADDR:
-            return "TCCR0";
-        default:
-            avr_error ("Bad address: 0x%04x", addr);
-    }
-    return NULL;
-}
-
 static int
 timer0_clk_incr_cb (uint64_t ck, AvrClass *data)
 {
@@ -473,10 +528,10 @@ timer0_clk_incr_cb (uint64_t ck, AvrClass *data)
     uint8_t last = timer->tcnt;
     TimerIntr_T *ti;
 
-    ti = (TimerIntr_T *)avr_core_get_vdev_by_name ((AvrCore *)
+    ti = (TimerIntr_T *)avr_core_get_vdev_by_addr ((AvrCore *)
                                                    vdev_get_core ((VDevice *)
                                                                   timer),
-                                                   "TimerIntr");
+                                                   timer->related_addr);
 
     if (timer->clk_cb == NULL)
         return CB_RET_REMOVE;
@@ -512,23 +567,36 @@ timer0_clk_incr_cb (uint64_t ck, AvrClass *data)
 
 /*@{*/
 
+static void timer16_add_addr (VDevice *vdev, int addr, char *name,
+                              int rel_addr, void *data);
 static void timer16_destroy (void *timer);
 static uint8_t timer16_read (VDevice *dev, int addr);
 static void timer16_write (VDevice *dev, int addr, uint8_t val);
 static void timer16_reset (VDevice *dev);
-static char *timer16_reg_name (VDevice *dev, int addr);
 static int timer16_clk_incr_cb (uint64_t time, AvrClass *data);
 static void timer16_handle_tccr_write (Timer16_T *timer);
 
 /** \brief Allocate a new 16 bit timer/counter. */
 
+VDevice *
+timer16_create (int addr, char *name, int rel_addr, void *data)
+{
+    uint8_t *def_data = (uint8_t *) data;
+    if (data)
+        return (VDevice *)timer16_new (addr, name, rel_addr,
+                                       global_timer16_defs[*def_data]);
+    else
+        avr_error ("Attempted timer 16 create with NULL data pointer");
+    return 0;
+}
+
 Timer16_T *
-timer16_new (Timer16Def timerdef)
+timer16_new (int addr, char *name, int rel_addr, Timer16Def timerdef)
 {
     Timer16_T *timer;
 
     timer = avr_new (Timer16_T, 1);
-    timer16_construct (timer, timerdef);
+    timer16_construct (timer, addr, name, rel_addr, timerdef);
     class_overload_destroy ((AvrClass *)timer, timer16_destroy);
 
     return timer;
@@ -537,22 +605,61 @@ timer16_new (Timer16Def timerdef)
 /** \brief Constructor for 16 bit timer/counter object. */
 
 void
-timer16_construct (Timer16_T *timer, Timer16Def timerdef)
+timer16_construct (Timer16_T *timer, int addr, char *name, int rel_addr,
+                   Timer16Def timerdef)
 {
-    char *name = "Timer16";
-
     if (timer == NULL)
         avr_error ("passed null ptr");
 
-    vdev_construct ((VDevice *)timer, name, TIMER16_BASE, TIMER16_SIZE,
-                    timer16_read, timer16_write, timer16_reset,
-                    timer16_reg_name);
+    vdev_construct ((VDevice *)timer, timer16_read, timer16_write,
+                    timer16_reset, timer16_add_addr);
 
     timer->timerdef = timerdef;
 
+    timer16_add_addr ((VDevice *)timer, addr, name, 0, NULL);
+    if (rel_addr)
+        timer->related_addr = rel_addr;
     timer16_reset ((VDevice *)timer);
 }
 
+static void
+timer16_add_addr (VDevice *vdev, int addr, char *name, int rel_addr,
+                  void *data)
+{
+    Timer16_T *ti = (Timer16_T *)vdev;
+
+    if (strncmp ("TCNTL", name, 5) == 0)
+    {
+        ti->tcntl_addr = addr;
+    }
+
+    else if (strncmp ("TCNTH", name, 5) == 0)
+    {
+        ti->tcnth_addr = addr;
+    }
+
+    else if (strncmp ("TCCRA", name, 5) == 0)
+    {
+        ti->tccra_addr = addr;
+    }
+
+    else if (strncmp ("TCCRB", name, 5) == 0)
+    {
+        ti->tccrb_addr = addr;
+    }
+
+    else if (strncmp ("TCCRC", name, 5) == 0)
+    {
+        ti->tccrc_addr = addr;
+    }
+
+    else
+    {
+        avr_error ("invalid Timer16 register name: '%s' @ 0x%04x", name,
+                   addr);
+    }
+}
+
 static void
 timer16_destroy (void *timer)
 {
@@ -567,21 +674,37 @@ timer16_read (VDevice *dev, int addr)
 {
     Timer16_T *timer = (Timer16_T *)dev;
 
-    switch (addr - vdev_get_base (dev))
+    if (addr == timer->tcntl_addr)
     {
-        case TCNTL_ADDR:
-            timer->TEMP = (uint8_t) ((timer->tcnt) >> 8);
-            return (timer->tcnt) & 0xFF;
-        case TCNTH_ADDR:
-            return timer->TEMP;
-        case TCCRA_ADDR:
-            return (timer->tccr) & 0xFF;
-        case TCCRB_ADDR:
-            return (timer->tccr) >> 8;
-            /* FIXME: Need TCCRC for mega128/64 and maybe others. */
-        default:
-            avr_error ("Bad address: 0x%04x", addr);
+        timer->TEMP = (uint8_t) ((timer->tcnt) >> 8);
+        return (timer->tcnt) & 0xFF;
     }
+
+    else if (addr == timer->tcnth_addr)
+    {
+        return timer->TEMP;
+    }
+
+    else if (addr == timer->tccra_addr)
+    {
+        return timer->tccra;
+    }
+
+    else if (addr == timer->tccrb_addr)
+    {
+        return timer->tccrb;
+    }
+
+    else if (addr == timer->tccrc_addr)
+    {
+        return timer->tccrc;
+    }
+
+    else
+    {
+        avr_error ("Bad address: 0x%04x", addr);
+    }
+
     return 0;                   /* will never get here */
 }
 
@@ -589,27 +712,38 @@ static void
 timer16_write (VDevice *dev, int addr, uint8_t val)
 {
     Timer16_T *timer = (Timer16_T *)dev;
-    int offset = addr - vdev_get_base (dev);
 
-    switch (offset)
+    if (addr == timer->tcntl_addr)
     {
-        case TCNTL_ADDR:
-            timer->tcnt = (((timer->TEMP) << 8) & 0xFF00) | val;
-            break;
-        case TCNTH_ADDR:
-            timer->TEMP = val;
-            break;
-        case TCCRA_ADDR:
-            timer->tccr = (timer->tccr & 0xFF00) | val;
-            timer16_handle_tccr_write (timer);
-            break;
-        case TCCRB_ADDR:
-            timer->tccr = (timer->tccr & 0xFF) | (val << 8);
-            timer16_handle_tccr_write (timer);
-            break;
-            /* FIXME: Need TCCRC for mega128/64 and maybe others. */
-        default:
-            avr_error ("Bad address: 0x%04x", addr);
+        timer->tcnt = (((timer->TEMP) << 8) & 0xFF00) | val;
+    }
+
+    else if (addr == timer->tcnth_addr)
+    {
+        timer->TEMP = val;
+    }
+
+    else if (addr == timer->tccra_addr)
+    {
+        timer->tccra = val;
+        timer16_handle_tccr_write (timer);
+    }
+
+    else if (addr == timer->tccrb_addr)
+    {
+        timer->tccrb = val;
+        timer16_handle_tccr_write (timer);
+    }
+
+    else if (addr == timer->tccrc_addr)
+    {
+        timer->tccrc = val;
+        timer16_handle_tccr_write (timer);
+    }
+
+    else
+    {
+        avr_error ("Bad address: 0x%04x", addr);
     }
 }
 
@@ -621,31 +755,14 @@ timer16_reset (VDevice *dev)
     timer->clk_cb = NULL;
     timer->ext_cb = NULL;
 
-    timer->tccr = 0;
+    timer->tccra = 0;
+    timer->tccrb = 0;
+    timer->tccrc = 0;
     timer->tcnt = 0;
 
     timer->divisor = 0;
 }
 
-static char *
-timer16_reg_name (VDevice *dev, int addr)
-{
-    switch (addr - vdev_get_base (dev))
-    {
-        case TCNTL_ADDR:
-            return "TCNT1L";
-        case TCNTH_ADDR:
-            return "TCNT1H";
-        case TCCRA_ADDR:
-            return "TCCR1A";
-        case TCCRB_ADDR:
-            return "TCCR1B";
-        default:
-            avr_error ("Bad address: 0x%04x", addr);
-    }
-    return NULL;
-}
-
 static void
 timer_intr_set_flag (TimerIntr_T *ti, uint8_t bitnr)
 {
@@ -660,11 +777,11 @@ timer16_clk_incr_cb (uint64_t ck, AvrClass *data)
 
     if (!timer->ti)
         timer->ti =
-            (TimerIntr_T *)avr_core_get_vdev_by_name ((AvrCore *)
+            (TimerIntr_T *)avr_core_get_vdev_by_addr ((AvrCore *)
                                                       vdev_get_core ((VDevice
                                                                       *)
                                                                      timer),
-                                                      "TimerIntr");
+                                                      timer->related_addr);
 
     if (timer->clk_cb == NULL)
         return CB_RET_REMOVE;
@@ -719,7 +836,7 @@ timer16_handle_tccr_write (Timer16_T *timer)
      * other can be installed at any given instant.
      */
 
-    cs = ((timer->tccr) >> 8) & 0x07;
+    cs = timer->tccrb & 0x07;
 
     switch (cs)
     {
@@ -777,23 +894,36 @@ timer16_handle_tccr_write (Timer16_T *timer)
 
 /*@{*/
 
+static void ocr_add_addr (VDevice *vdev, int addr, char *name, int rel_addr,
+                          void *data);
 static void ocreg16_destroy (void *ocr);
 static uint8_t ocreg16_read (VDevice *dev, int addr);
 static void ocreg16_write (VDevice *dev, int addr, uint8_t val);
 static void ocreg16_reset (VDevice *dev);
-static char *ocreg16_reg_name (VDevice *dev, int addr);
 
 /** \brief Allocate a new 16 bit Output Compare Register
   * \param ocrdef The definition struct for the \a OCR to be created
   */
 
+VDevice *
+ocreg16_create (int addr, char *name, int rel_addr, void *data)
+{
+    uint8_t *def_data = (uint8_t *) data;
+    if (data)
+        return (VDevice *)ocreg16_new (addr, name,
+                                       global_ocreg16_defs[*def_data]);
+    else
+        avr_error ("Attempted OCReg create with NULL data pointer");
+    return 0;
+}
+
 OCReg16_T *
-ocreg16_new (OCReg16Def ocrdef)
+ocreg16_new (int addr, char *name, OCReg16Def ocrdef)
 {
     OCReg16_T *ocreg;
 
     ocreg = avr_new (OCReg16_T, 1);
-    ocreg16_construct (ocreg, ocrdef);
+    ocreg16_construct (ocreg, addr, name, ocrdef);
     class_overload_destroy ((AvrClass *)ocreg, ocreg16_destroy);
 
     return ocreg;
@@ -802,20 +932,46 @@ ocreg16_new (OCReg16Def ocrdef)
 /** \brief Constructor for 16 bit Output Compare Register object. */
 
 void
-ocreg16_construct (OCReg16_T *ocreg, OCReg16Def ocrdef)
+ocreg16_construct (OCReg16_T *ocreg, int addr, char *name, OCReg16Def ocrdef)
 {
     if (ocreg == NULL)
         avr_error ("passed null ptr");
 
-    vdev_construct ((VDevice *)ocreg, ocrdef.ocrdev_name, ocrdef.base,
-                    OCREG16_SIZE, ocreg16_read, ocreg16_write, ocreg16_reset,
-                    ocreg16_reg_name);
+    vdev_construct ((VDevice *)ocreg, ocreg16_read, ocreg16_write,
+                    ocreg16_reset, ocr_add_addr);
 
     ocreg->ocrdef = ocrdef;
 
+    ocr_add_addr ((VDevice *)ocreg, addr, name, 0, NULL);
     ocreg16_reset ((VDevice *)ocreg);
 }
 
+static void
+ocr_add_addr (VDevice *vdev, int addr, char *name, int rel_addr, void *data)
+{
+    OCReg16_T *ocreg = (OCReg16_T *)vdev;
+
+    if ((strncmp ("OCRAL", name, 5) == 0) || (strncmp ("OCRBL", name, 5) == 0)
+        || (strncmp ("OCRCL", name, 5) == 0))
+    {
+        ocreg->ocrl_addr = addr;
+    }
+
+    else if ((strncmp ("OCRAH", name, 5) == 0)
+             || (strncmp ("OCRBH", name, 5) == 0)
+             || (strncmp ("OCRCH", name, 5) == 0))
+
+    {
+        ocreg->ocrh_addr = addr;
+    }
+
+    else
+    {
+        avr_error ("invalid Timer16 register name: '%s' @ 0x%04x", name,
+                   addr);
+    }
+}
+
 static void
 ocreg16_destroy (void *ocreg)
 {
@@ -830,15 +986,21 @@ ocreg16_read (VDevice *dev, int addr)
 {
     OCReg16_T *ocreg = (OCReg16_T *)dev;
 
-    switch (addr - vdev_get_base (dev))
+    if (addr == ocreg->ocrl_addr)
     {
-        case OCRL_ADDR:
-            return (ocreg->ocr) & 0xFF;
-        case OCRH_ADDR:
-            return (ocreg->ocr) >> 8;
-        default:
-            avr_error ("Bad address: 0x%04x", addr);
+        return (ocreg->ocr) & 0xFF;
     }
+
+    else if (addr == ocreg->ocrh_addr)
+    {
+        return (ocreg->ocr) >> 8;
+    }
+
+    else
+    {
+        avr_error ("Bad address: 0x%04x", addr);
+    }
+
     return 0;
 }
 
@@ -847,16 +1009,19 @@ ocreg16_write (VDevice *dev, int addr, uint8_t val)
 {
     OCReg16_T *ocreg = (OCReg16_T *)dev;
 
-    switch (addr - vdev_get_base (dev))
+    if (addr == ocreg->ocrl_addr)
     {
-        case OCRL_ADDR:
-            ocreg->ocr = (((ocreg->TEMP) << 8) & 0xFF00) | val;
-            break;
-        case OCRH_ADDR:
-            ocreg->TEMP = val;
-            break;
-        default:
-            avr_error ("Bad address: 0x%04x", addr);
+        ocreg->ocr = (((ocreg->TEMP) << 8) & 0xFF00) | val;
+    }
+
+    else if (addr == ocreg->ocrh_addr)
+    {
+        ocreg->TEMP = val;
+    }
+
+    else
+    {
+        avr_error ("Bad address: 0x%04x", addr);
     }
 }
 
@@ -868,21 +1033,4 @@ ocreg16_reset (VDevice *dev)
     ocreg->ocr = 0;
 }
 
-static char *
-ocreg16_reg_name (VDevice *dev, int addr)
-{
-    OCReg16_T *ocreg = (OCReg16_T *)dev;
-
-    switch (addr - vdev_get_base (dev))
-    {
-        case OCRL_ADDR:
-            return ocreg->ocrdef.ocrl_name;
-        case OCRH_ADDR:
-            return ocreg->ocrdef.ocrh_name;
-        default:
-            avr_error ("Bad address: 0x%04x", addr);
-    }
-    return 0;
-}
-
 /*@}*/
