@@ -5,7 +5,8 @@
 
 void func4 (void)
 {
-    PORTC |= _BV(PC0);
+    static uint8_t val;
+    PORTC = val++;
 }
 
 void func3 (void)
@@ -25,7 +26,12 @@ void func1 (void)
 
 int main(void)
 {
-    func1();
+    DDRC = 0xff;
+
+    while (1)
+    {
+        func1();
+    }
 
     return 0;
 }
